@@ -32,7 +32,6 @@
 (require 'tablist)
 (require 'cl-lib)
 
-
 
 ;; * ================================================================== *
 ;; * Customizations
@@ -57,23 +56,86 @@ Implement and describe basic org example."
   :group 'pdf-annot
   :type 'hook)
 
-(defcustom pdf-annot-default-text-annotation-properties
-  `((icon . "Note")
-    (color . "#ff0000")
-    (label . ,user-full-name)
-    (popup-is-open . nil))
-  "Alist of initial properties for new text annotations.
+(defcustom pdf-annot-default-text-annotation-properties nil
+  "Alist of initial properties for new text annotations."
+  :group 'pdf-annot
+  :type '(alist :key-type symbol :value-type sexp))
 
-FIXME: Describe. Restrict keys and values."
-  :group 'pdf-annot)
+(defcustom pdf-annot-default-markup-annotation-properties nil
+  "Alist of initial properties for new markup annotations."
+  :group 'pdf-annot
+  :type '(alist :key-type symbol :value-type sexp))
 
-(defcustom pdf-annot-default-markup-annotation-properties
-  `((label . ,user-full-name)
-    (popup-is-open . nil))
-  "Alist of initial properties for new markup annotations.
+(make-obsolete-variable 'pdf-annot-default-text-annotation-properties
+                        'pdf-annot-default-annotation-properties
+                        "0.90")
 
-FIXME: Describe. Restrict keys and values."
-  :group 'pdf-annot)
+(make-obsolete-variable 'pdf-annot-default-markup-annotation-properties
+                        'pdf-annot-default-annotation-properties
+                        "0.90")
+
+(defcustom pdf-annot-default-annotation-properties
+  `((t (label . ,user-full-name))
+    (text (icon . "Note")
+          (color . "#ff0000"))
+    (highlight (color . "yellow"))
+    (squiggly (color . "orange"))
+    (strike-out(color . "red"))
+    (underline (color . "blue")))
+  "An alist of initial properties for new annotations.
+
+The alist contains a sub-alist for each of the currently available
+annotation types, i.e. text, highlight, squiggly, strike-out and
+underline.  Additionally a sub-alist with a key of t acts as a default
+entry.
+
+Each of these sub-alists contain default property-values of newly
+added annotations of its respective type.
+
+Some of the most important properties and their types are label
+\(a string\), contents \(a string\), color \(a color\) and, for
+text-annotations only, icon \(one of the standard icon-types, see
+`pdf-annot-standard-text-icons'\).
+
+For example a value of
+
+  \(\(t \(color . \"red\"\)
+      \(label . \"Joe\"\)
+   \(highlight \(color . \"green\"\)\)
+
+would use a green color for highlight and a red one for other
+annotations.  Additionally the label for all annotations is set
+to \"Joe\"."
+
+  :group 'pdf-annot
+  :type (let* ((label '(cons :tag "Label" (const label) string))
+               (contents '(cons :tag "Contents" (const contents) string))
+               (color '(cons :tag "Color" (const color) color))
+               (icon `(cons :tag "Icon"
+                            (const icon)
+                            (choice
+                             ,@(mapcar (lambda (icon)
+                                         `(const ,icon))
+                                       '("Note" "Comment" "Key" "Help" "NewParagraph"
+                                         "Paragraph" "Insert" "Cross" "Circle")))))
+               (other '(repeat
+                        :tag "Other properties"
+                        (cons :tag "Property"
+                              (symbol :tag "Key  ")
+                              (sexp :tag "Value"))))
+               (text-properties
+                `(set ,label ,contents ,color ,icon ,other))
+               (markup-properties
+                `(set ,label ,contents ,color))
+               (all-properties
+                `(set ,label ,contents ,color ,icon ,other)))
+          `(set
+            (cons :tag "All Annotations" (const t) ,all-properties)
+            (cons :tag "Text Annotations" (const text) ,text-properties)
+            (cons :tag "Highlight Annotations" (const highlight) ,markup-properties)
+            (cons :tag "Underline Annotations" (const underline) ,markup-properties)
+            (cons :tag "Squiggly Annotations" (const squiggly) ,markup-properties)
+            (cons :tag "Strike-out Annotations" (const strike-out) ,markup-properties))))
 
 (defcustom pdf-annot-print-annotation-functions
   '(pdf-annot-print-annotation-latex-maybe)
@@ -118,7 +180,8 @@ order to display text properties;
 
 `tooltip-hide-delay' is set to infinity, in order to not being
 annoyed while reading the annotations."
-  :group 'pdf-annot)
+  :group 'pdf-annot
+  :type 'boolean)
 
 (defcustom pdf-annot-activate-created-annotations nil
   "Whether to activate (i.e. edit) created annotations."
@@ -978,7 +1041,12 @@ Return the new annotation."
                    page
                    (if (eq type 'text)
                        (car edges)
-                     '(0 0 0 0))
+                     (apply #'pdf-util-edges-union
+                            (apply #'append
+                                   (mapcar
+                                    (lambda (e)
+                                      (pdf-info-getselection page e))
+                                    edges))))
                    type
                    nil
                    (if (not (eq type 'text)) edges)))
@@ -1048,6 +1116,8 @@ Return the new annotation."
         (and icon `((icon . ,icon)))
         property-alist
         pdf-annot-default-text-annotation-properties
+        (cdr (assq 'text pdf-annot-default-annotation-properties))
+        (cdr (assq t pdf-annot-default-annotation-properties))
         `((color . ,(car pdf-annot-color-history))))))))
 
 (defun pdf-annot-mouse-add-text-annotation (ev)
@@ -1093,6 +1163,8 @@ Return the new annotation."
     (and color `((color . ,color)))
     property-alist
     pdf-annot-default-markup-annotation-properties
+    (cdr (assq type pdf-annot-default-annotation-properties))
+    (cdr (assq t pdf-annot-default-annotation-properties))
     (when pdf-annot-color-history
       `((color . ,(car pdf-annot-color-history))))
     '((color . "#ffff00")))
@@ -1115,7 +1187,7 @@ See also `pdf-annot-add-markup-annotation'."
   (pdf-annot-add-markup-annotation list-of-edges 'underline color property-alist))
 
 (defun pdf-annot-add-strikeout-markup-annotation (list-of-edges
-                                                 &optional color property-alist)
+                                                  &optional color property-alist)
   "Add a new strike-out annotation in the selected window.
 
 See also `pdf-annot-add-markup-annotation'."
@@ -1123,7 +1195,7 @@ See also `pdf-annot-add-markup-annotation'."
   (pdf-annot-add-markup-annotation list-of-edges 'strike-out color property-alist))
 
 (defun pdf-annot-add-highlight-markup-annotation (list-of-edges
-                                                 &optional color property-alist)
+                                                  &optional color property-alist)
   "Add a new highlight annotation in the selected window.
 
 See also `pdf-annot-add-markup-annotation'."
@@ -1401,13 +1473,13 @@ annotation's contents and otherwise `text-mode'. "
               (pdf-annot-edit-contents-minor-mode 1)
               (current-buffer))))
     (with-current-buffer pdf-annot-edit-contents--buffer
-      (setq pdf-annot-edit-contents--annotation a)
-      (funcall pdf-annot-edit-contents-setup-function a)
       (let ((inhibit-read-only t))
         (erase-buffer)
         (save-excursion (insert (pdf-annot-get a 'contents)))
-        (set-buffer-modified-p nil)
-        (current-buffer)))))
+        (set-buffer-modified-p nil))
+      (setq pdf-annot-edit-contents--annotation a)
+      (funcall pdf-annot-edit-contents-setup-function a)
+      (current-buffer))))
 
 (defun pdf-annot-edit-contents (a)
   (select-window
@@ -1434,8 +1506,34 @@ annotation's contents and otherwise `text-mode'. "
      display-buffer-pop-up-window)
     (inhibit-same-window . t))
   "Display action used when displaying the list buffer."
+  :group 'pdf-annot
+  :type display-buffer--action-custom-type)
+
+(defcustom pdf-annot-list-format
+  '((page . 3)
+    (type . 10)
+    (label . 24)
+    (date . 24))
+  "Annotation properties visible in the annotation list.
+
+It should be a list of \(PROPERTIZE. WIDTH\), where PROPERTY is a
+symbol naming one of supported properties to list and WIDTH its
+desired column-width.
+
+Currently supported properties are page, type, label, date and contents."
+  :type '(alist :key-type (symbol))
+  :options '((page (integer :value 3 :tag "Column Width"))
+             (type (integer :value 10 :tag "Column Width" ))
+             (label (integer :value 24 :tag "Column Width"))
+             (date (integer :value 24 :tag "Column Width"))
+             (contents (integer :value 56 :tag "Column Width")))
   :group 'pdf-annot)
 
+(defcustom pdf-annot-list-highlight-type nil
+  "Whether to highlight \"Type\" column annotation list with annotation color."
+  :group 'pdf-annot
+  :type 'boolean)
+
 (defvar-local pdf-annot-list-buffer nil)
 
 (defvar-local pdf-annot-list-document-buffer nil)
@@ -1482,33 +1580,70 @@ belong to the same page and A1 is displayed above/left of A2."
                                      pdf-annot-list-document-buffer)
                 'pdf-annot-compare-annotations)))
 
+(defun pdf-annot--make-entry-formatter (a)
+  (lambda (fmt)
+    (let ((entry-type (car fmt))
+          (entry-width (cdr fmt))
+          ;; Taken from css-mode.el
+          (contrasty-color
+           (lambda (name)
+             (if (> (color-distance name "black") 292485)
+                 "black" "white")))
+          (prune-newlines
+           (lambda (str)
+             (replace-regexp-in-string "\n" " " str t t))))
+      (cl-ecase entry-type
+        (date (pdf-annot-print-property a 'modified))
+        (page (pdf-annot-print-property a 'page))
+        (label (funcall prune-newlines
+                        (pdf-annot-print-property a 'label)))
+        (contents
+         (truncate-string-to-width
+          (funcall prune-newlines
+                   (pdf-annot-print-property a 'contents))
+          entry-width))
+        (type
+         (let ((color (pdf-annot-get a 'color))
+               (type (pdf-annot-print-property a 'type)))
+           (if pdf-annot-list-highlight-type
+               (propertize
+                type 'face
+                `(:background ,color
+                  :foreground ,(funcall contrasty-color color)))
+             type)))))))
+
 (defun pdf-annot-list-create-entry (a)
   "Create a `tabulated-list-entries' entry for annotation A."
   (list (pdf-annot-get-id a)
-        (vector
-         (pdf-annot-print-property a 'page)
-         (pdf-annot-print-property a 'type)
-         (replace-regexp-in-string
-          "\n" " "
-          (pdf-annot-print-property a 'label) t t)
-         (if (pdf-annot-get a 'modified)
-             (pdf-annot-print-property a 'modified)
-           (if (pdf-annot-get a 'created)
-               (pdf-annot-print-property a 'created)
-             "Unknown date")))))
+        (vconcat
+         (mapcar (pdf-annot--make-entry-formatter a)
+                 pdf-annot-list-format))))
 
 (define-derived-mode pdf-annot-list-mode tablist-mode "Annots"
-  (setq tabulated-list-entries 'pdf-annot-list-entries
-        tabulated-list-format (vector
-                               '("Pg." 3 t :read-only t :right-align t)
-                               `("Type" 10 t :read-only t)
-                               `("Label" 24 t :read-only t)
-                               '("Date" 24 t :read-only t))
-        tabulated-list-padding 2)
+  (let* ((page-sorter
+          (lambda (a b)
+            (< (string-to-number (aref (cadr a) 0))
+               (string-to-number (aref (cadr b) 0)))))
+         (format-generator
+          (lambda (format)
+            (let ((field (car format))
+                  (width (cdr format)))
+              (cl-case field
+                (page `("Pg." 3 ,page-sorter :read-only t :right-alight t))
+                (t (list
+                    (capitalize (symbol-name field))
+                    width t :read-only t)))))))
+    (setq tabulated-list-entries 'pdf-annot-list-entries
+          tabulated-list-format (vconcat
+                                 (mapcar
+                                  format-generator
+                                  pdf-annot-list-format))
+          tabulated-list-padding 2))
   (set-keymap-parent pdf-annot-list-mode-map tablist-mode-map)
   (use-local-map pdf-annot-list-mode-map)
-  (setq tablist-current-filter
-        `(not (== "Type" "link")))
+  (when (assq 'type pdf-annot-list-format)
+    (setq tablist-current-filter
+          `(not (== "Type" "link"))))
   (tabulated-list-init-header))
 
 (defun pdf-annot-list-annotations ()
