@@ -63,18 +63,30 @@
 
 (defcustom pdf-info-epdfinfo-program
   (let ((executable (if (eq system-type 'windows-nt)
-                        "epdfinfo.exe"
-                      "epdfinfo")))
-    (or (executable-find executable)
-        (expand-file-name
-         executable
-         (let ((directory (file-name-directory (or load-file-name default-directory))))
-           (cl-find-if 'file-exists-p
-                       `(,(expand-file-name "../server" directory)
-                         ,directory))))))
+                        "epdfinfo.exe" "epdfinfo"))
+        (default-directory
+          (or (and load-file-name
+                   (file-name-directory load-file-name))
+              default-directory)))
+    (cl-labels ((try-directory (directory)
+                  (and (file-directory-p directory)
+                       (file-executable-p (expand-file-name executable directory))
+                       (expand-file-name executable directory))))
+      (or (executable-find executable)
+          ;; This works if epdfinfo is in the same place as emacs and
+          ;; the editor was started with an absolute path, i.e. it is
+          ;; ment for Windows/Msys2.
+          (and (stringp (car-safe command-line-args))
+               (file-name-directory (car command-line-args))
+               (try-directory
+                (file-name-directory (car command-line-args))))
+          ;; If we are running directly from the git repo.
+          (try-directory (expand-file-name "../server"))
+          ;; Fall back to epdfinfo in the directory of this file.
+          (expand-file-name executable))))
   "Filename of the epdfinfo executable."
   :group 'pdf-info
-  :type '(file :must-match t))
+  :type 'file)
 
 (defcustom pdf-info-epdfinfo-error-filename nil
   "Filename for error output of the epdfinfo executable.
@@ -120,7 +132,9 @@ is set to nil."
   "A hook ran after a document was closed in the server.
 
 The hook is run in the documents buffer, if it exists. Otherwise
-in a `with-temp-buffer' form.")
+in a `with-temp-buffer' form."
+  :group 'pdf-info
+  :type 'hook)
 
 
 
@@ -188,6 +202,16 @@ server, that it never ran.")
 ;; * Process handling
 ;; * ================================================================== *
 
+(defconst pdf-info-empty-page-data
+  (eval-when-compile
+    (concat
+     "%PDF-1.0\n1 0 obj<</Type/Catalog/Pages 2 0 R>>endobj 2 0"
+     " obj<</Type/Pages/Kids[3 0 R]/Count 1>>endobj 3 0 obj<</"
+     "Type/Page/MediaBox[0 0 3 3]>>endobj\nxref\n0 4\n00000000"
+     "0065535 f\n0000000010 00000 n\n0000000053 00000 n\n00000"
+     "00102 00000 n\ntrailer<</Size 4/Root 1 0 R>>\nstartxref\n149\n%EOF"))
+  "PDF data of an empty page.")
+
 (defun pdf-info-process ()
   "Return the process object or nil."
   (and pdf-info--queue
@@ -195,6 +219,13 @@ server, that it never ran.")
        (tq-process pdf-info--queue)))
 
 (defun pdf-info-check-epdfinfo (&optional interactive-p)
+  "Check if the server should be working properly.
+
+Signal an error if some problem was found.  Message a
+confirmation, if INTERACTIVE-P is non-nil and no problems were
+found.
+
+Returns nil."
   (interactive "p")
   (let ((executable pdf-info-epdfinfo-program))
     (unless (stringp executable)
@@ -202,23 +233,36 @@ server, that it never ran.")
     (unless (file-executable-p executable)
       (error "pdf-info-epdfinfo-program is not executable"))
     (when pdf-info-epdfinfo-error-filename
-      (unless (file-writable-p pdf-info-epdfinfo-error-filename)
-        (error "pdf-info-epdfinfo-error-filename should be nil or a writable filename")))
-    (let ((tempfile (make-temp-file "pdf-info-check-epdfinfo"))
-          (default-directory "~"))
+      (unless (and (stringp pdf-info-epdfinfo-error-filename)
+                   (file-writable-p pdf-info-epdfinfo-error-filename))
+        (error "pdf-info-epdfinfo-error-filename should contain writable filename")))
+    (let* ((default-directory (expand-file-name "~/"))
+           (cmdfile (make-temp-file "commands"))
+           (pdffile (make-temp-file "empty.pdf"))
+           (tempdir (make-temp-file "tmpdir" t))
+           (process-environment (cons (concat "TMPDIR=" tempdir)
+                                      process-environment)))
       (unwind-protect
           (with-temp-buffer
-            (with-temp-file tempfile
-              (insert "quit\n"))
+            (with-temp-file pdffile
+              (set-buffer-multibyte nil)
+              (insert pdf-info-empty-page-data))
+            (with-temp-file cmdfile
+              (insert (format "renderpage:%s:1:100\nquit\n"
+                              (pdf-info-query--escape pdffile))))
             (unless (= 0 (apply #'call-process
-                                executable tempfile (current-buffer)
+                                executable cmdfile (current-buffer)
                                 nil (when pdf-info-epdfinfo-error-filename
                                       (list pdf-info-epdfinfo-error-filename))))
               (error "Error running `%s': %s"
                      pdf-info-epdfinfo-program
                      (buffer-string))))
-        (when (file-exists-p tempfile)
-          (delete-file tempfile)))))
+        (when (file-exists-p cmdfile)
+          (delete-file cmdfile))
+        (when (file-exists-p pdffile)
+          (delete-file pdffile))
+        (when (file-exists-p tempdir)
+          (delete-directory tempdir t)))))
   (when interactive-p
     (message "The epdfinfo program appears to be working."))
   nil)
@@ -338,7 +382,8 @@ error."
         (accept-process-output (pdf-info-process) 0.01))
       (when (and (not done)
                  (not (eq (process-status (pdf-info-process))
-                          'run)))
+                          'run))
+                 (not (eq cmd 'quit)))
         (error "The epdfinfo server quit unexpectedly."))
       (cond
        ((null status) response)
